function [theta v] = our_method(Phi,Y,ratio_of_lambda_max)
%our_method computes a segmented ARX-model using sum-of-norms 
%regularization (Euclidean norm supported in this version of the code). 
%
%CALLING SEQUENCES:
% [theta v] = our_method(Phi,Y,ratio_of_lambda_max)
%
%INPUT:
% Phi                : Nxn dim regressor matrix, n dim of regressors  
% Y                  : Nx1 dim output 
% ratio_of_lambda_max: Scalar design parameter. Regularization parmeter
%                      lambda = lambda_max*ratio_of_lambda_max.
%
%OUTPUT:
% theta              : 4xnxN dim tensor of ARX parmeters. The first 
%                      dimension can be used to choose refinement level. 
%                      theta(4,:,:) contains the final estimate of the ARX 
%                      parmeters.
% v                  : 4xN dim matrix containing residuals. v(4,:) final
%                      residuals.
%
% our_method.m uses comp_lambdamax.m and segreg_sumofnorms.m. CVX is also
% needed.

% AUTHOR    Henrik Ohlsson <ohlsson@isy.liu.se>
% UPDATE    Dec 1 2009
%
% COPYRIGHT 2009 Henrik Ohlsson, Lennart Ljung and Stephen Boyd

[N n] = size(Phi);
no_of_refinments_iter = 2;
theta = zeros(no_of_refinments_iter+2,n,N);
v = zeros(no_of_refinments_iter+2,N-1);
e = 0.01;

% compute lambda_max
lambda_max = comp_lambdamax(Phi,Y)

% compute theta using lambda_max
w = ones(length(Y)-1,1);
[theta(1,:,:) v(1,:)] = segreg_sumofnorms(Phi,Y,lambda_max*ratio_of_lambda_max,w);

% refinement #1: iterative refinement
for i = 1:no_of_refinments_iter
w = 1./(e+v(i,:));
[theta(i+1,:,:) v(i+1,:)] = segreg_sumofnorms(Phi,Y,lambda_max*ratio_of_lambda_max,w);
end

% refinement #2
w = v(i+1,:);
w(v(i+1,:)<0.0005)=100000;
w(v(i+1,:)>0.0005)=0;
[theta(i+2,:,:) v(i+2,:)] = segreg_sumofnorms(Phi,Y,lambda_max*ratio_of_lambda_max,w);

